@@ -39,11 +39,6 @@ pub enum Error {
         source: serde_json::Error,
     },
 
-    #[error("{source}")]
-    FtdRT {
-        #[from]
-        source: ftd_rt::Error,
-    },
     #[error("{source}")]
     Failure {
         #[from]
