@@ -2,15 +2,15 @@
 pub struct Variable {
     pub name: String,
     pub value: Value,
-    pub conditions: Vec<(crate::p2::Boolean, crate::Value)>,
+    pub conditions: Vec<(ftd::p2::Boolean, ftd::Value)>,
 }
 
 #[derive(Debug, Clone, PartialEq, serde::Serialize, serde::Deserialize)]
 #[serde(tag = "type")]
 pub enum PropertyValue {
-    Value { value: crate::variable::Value },
-    Reference { name: String, kind: crate::p2::Kind },
-    Variable { name: String, kind: crate::p2::Kind },
+    Value { value: ftd::variable::Value },
+    Reference { name: String, kind: ftd::p2::Kind },
+    Variable { name: String, kind: ftd::p2::Kind },
 }
 
 impl PropertyValue {
@@ -24,6 +24,16 @@ impl PropertyValue {
     ) -> ftd::p1::Result<ftd::PropertyValue> {
         let property_type = if let Some(arg) = value.strip_prefix('$') {
             PropertyType::Variable(arg.to_string())
+        } else if let Some(ftd::p2::Kind::UI { .. }) = expected_kind {
+            if !value.contains(':') {
+                return ftd::e2(
+                    format!("expected `:`, found: `{}`", value),
+                    doc.name,
+                    line_number,
+                );
+            }
+            let (name, caption) = ftd::p2::utils::split(value.to_string(), ":")?;
+            PropertyType::Component { name, caption }
         } else {
             let value = if let Some(value) = value.strip_prefix('\\') {
                 value.to_string()
@@ -36,7 +46,7 @@ impl PropertyValue {
         let (part1, part2) = get_parts(&property_type.string())?;
 
         return Ok(match property_type {
-            PropertyType::Variable(string) => {
+            PropertyType::Variable(string) | PropertyType::Component { name: string, .. } => {
                 let (kind, is_doc) = match arguments.get(&part1) {
                     _ if part1.eq("MOUSE-IN") => (
                         ftd::p2::Kind::Boolean {
@@ -44,9 +54,12 @@ impl PropertyValue {
                         },
                         false,
                     ),
-                    None => match doc.get_value(line_number, &string) {
-                        Ok(val) => (val.kind(), true),
-                        Err(e) => {
+                    None => match doc.get_thing(line_number, &string) {
+                        Ok(ftd::p2::Thing::Variable(v)) => (v.value.kind(), true),
+                        Ok(ftd::p2::Thing::Component(_)) => {
+                            (ftd::p2::Kind::UI { default: None }, true)
+                        }
+                        e => {
                             return ftd::e2(
                                 format!("{} is not present in doc, {:?}", part1, e),
                                 doc.name,
@@ -58,7 +71,6 @@ impl PropertyValue {
                 };
 
                 let found_kind = get_kind(line_number, &kind, part2, doc, &expected_kind)?;
-
                 if is_doc {
                     PropertyValue::Reference {
                         name: doc
@@ -83,10 +95,10 @@ impl PropertyValue {
                 }
                 let expected_kind = expected_kind.unwrap();
                 match expected_kind.inner() {
-                    ftd::p2::Kind::Integer { .. } => crate::PropertyValue::Value {
-                        value: crate::Value::Integer {
+                    ftd::p2::Kind::Integer { .. } => ftd::PropertyValue::Value {
+                        value: ftd::Value::Integer {
                             value: string.parse::<i64>().map_err(|e| {
-                                crate::p1::Error::ParseError {
+                                ftd::p1::Error::ParseError {
                                     message: e.to_string(),
                                     doc_id: doc.name.to_string(),
                                     line_number,
@@ -94,10 +106,10 @@ impl PropertyValue {
                             })?,
                         },
                     },
-                    ftd::p2::Kind::Decimal { .. } => crate::PropertyValue::Value {
-                        value: crate::Value::Decimal {
+                    ftd::p2::Kind::Decimal { .. } => ftd::PropertyValue::Value {
+                        value: ftd::Value::Decimal {
                             value: string.parse::<f64>().map_err(|e| {
-                                crate::p1::Error::ParseError {
+                                ftd::p1::Error::ParseError {
                                     message: e.to_string(),
                                     doc_id: doc.name.to_string(),
                                     line_number,
@@ -105,10 +117,10 @@ impl PropertyValue {
                             })?,
                         },
                     },
-                    ftd::p2::Kind::Boolean { .. } => crate::PropertyValue::Value {
-                        value: crate::Value::Boolean {
+                    ftd::p2::Kind::Boolean { .. } => ftd::PropertyValue::Value {
+                        value: ftd::Value::Boolean {
                             value: string.parse::<bool>().map_err(|e| {
-                                crate::p1::Error::ParseError {
+                                ftd::p1::Error::ParseError {
                                     message: e.to_string(),
                                     doc_id: doc.name.to_string(),
                                     line_number,
@@ -116,8 +128,8 @@ impl PropertyValue {
                             })?,
                         },
                     },
-                    ftd::p2::Kind::String { .. } => crate::PropertyValue::Value {
-                        value: crate::Value::String {
+                    ftd::p2::Kind::String { .. } => ftd::PropertyValue::Value {
+                        value: ftd::Value::String {
                             text: string,
                             source: source.unwrap_or(ftd::TextSource::Header),
                         },
@@ -137,12 +149,15 @@ impl PropertyValue {
         enum PropertyType {
             Value(String),
             Variable(String),
+            Component { name: String, caption: String },
         }
 
         impl PropertyType {
             fn string(&self) -> String {
                 match self {
-                    PropertyType::Value(s) | PropertyType::Variable(s) => s.to_string(),
+                    PropertyType::Value(s)
+                    | PropertyType::Variable(s)
+                    | PropertyType::Component { name: s, .. } => s.to_string(),
                 }
             }
         }
@@ -162,7 +177,7 @@ impl PropertyValue {
             p2: Option<String>,
             doc: &ftd::p2::TDoc,
             expected_kind: &Option<ftd::p2::Kind>,
-        ) -> crate::p1::Result<ftd::p2::Kind> {
+        ) -> ftd::p1::Result<ftd::p2::Kind> {
             let mut found_kind = kind.to_owned();
             if let ftd::p2::Kind::Record { ref name } = kind {
                 if let Some(p2) = p2 {
@@ -199,7 +214,7 @@ impl PropertyValue {
         }
     }
 
-    pub fn kind(&self) -> crate::p2::Kind {
+    pub fn kind(&self) -> ftd::p2::Kind {
         match self {
             Self::Value { value: v } => v.kind(),
             Self::Reference { kind, .. } => kind.to_owned(),
@@ -209,22 +224,22 @@ impl PropertyValue {
     pub fn resolve(
         &self,
         line_number: usize,
-        arguments: &std::collections::BTreeMap<String, crate::Value>,
-        doc: &crate::p2::TDoc,
-    ) -> crate::p1::Result<Value> {
+        arguments: &std::collections::BTreeMap<String, ftd::Value>,
+        doc: &ftd::p2::TDoc,
+    ) -> ftd::p1::Result<Value> {
         self.resolve_with_root(line_number, arguments, doc, None)
     }
 
     pub fn resolve_with_root(
         &self,
         line_number: usize,
-        arguments: &std::collections::BTreeMap<String, crate::Value>,
-        doc: &crate::p2::TDoc,
+        arguments: &std::collections::BTreeMap<String, ftd::Value>,
+        doc: &ftd::p2::TDoc,
         root_name: Option<&str>,
-    ) -> crate::p1::Result<Value> {
+    ) -> ftd::p1::Result<Value> {
         Ok(match self {
-            crate::PropertyValue::Value { value: v } => v.to_owned(),
-            crate::PropertyValue::Variable {
+            ftd::PropertyValue::Value { value: v } => v.to_owned(),
+            ftd::PropertyValue::Variable {
                 name,
                 kind: argument_kind,
             } => {
@@ -284,17 +299,21 @@ impl PropertyValue {
                     }
                 }
             }
-            crate::PropertyValue::Reference {
+            ftd::PropertyValue::Reference {
                 name: reference_name,
                 kind: reference_kind,
             } => {
                 assert_eq!(self.kind(), *reference_kind);
-                let (default, condition) = match doc
-                    .
-                        get_value_and_conditions_with_root(0, reference_name.as_str(), root_name) //todo
+                let (default, condition) = if let Ok(d) =
+                    doc.get_value_and_conditions_with_root(0, reference_name.as_str(), root_name)
                 {
-                    Ok(d) => d,
-                    _ => return reference_kind.to_value(line_number, doc.name),
+                    d
+                } else if let Ok(d) =
+                    doc.get_component_with_root(0, reference_name.as_str(), root_name)
+                {
+                    return d.to_value(reference_kind);
+                } else {
+                    return reference_kind.to_value(line_number, doc.name);
                 };
                 let mut value = default;
                 for (boolean, property) in condition {
@@ -322,7 +341,7 @@ pub enum TextSource {
 #[allow(clippy::large_enum_variant)]
 pub enum Value {
     None {
-        kind: crate::p2::Kind,
+        kind: ftd::p2::Kind,
     },
     String {
         text: String,
@@ -348,11 +367,16 @@ pub enum Value {
     },
     List {
         data: Vec<Value>,
-        kind: crate::p2::Kind,
+        kind: ftd::p2::Kind,
     },
     Map {
         data: std::collections::BTreeMap<String, Value>,
+        kind: ftd::p2::Kind,
+    },
+    UI {
+        name: String,
         kind: crate::p2::Kind,
+        data: std::collections::BTreeMap<String, ftd::component::Property>,
     },
 }
 
@@ -370,29 +394,30 @@ impl Value {
         false
     }
 
-    pub fn kind(&self) -> crate::p2::Kind {
+    pub fn kind(&self) -> ftd::p2::Kind {
         match self {
             Value::None { kind: k } => k.to_owned(),
-            Value::String { source, .. } => crate::p2::Kind::String {
+            Value::String { source, .. } => ftd::p2::Kind::String {
                 caption: *source == TextSource::Caption,
                 body: *source == TextSource::Body,
                 default: None,
             },
-            Value::Integer { .. } => crate::p2::Kind::integer(),
-            Value::Decimal { .. } => crate::p2::Kind::decimal(),
-            Value::Boolean { .. } => crate::p2::Kind::boolean(),
-            Value::Record { name: id, .. } => crate::p2::Kind::Record {
+            Value::Integer { .. } => ftd::p2::Kind::integer(),
+            Value::Decimal { .. } => ftd::p2::Kind::decimal(),
+            Value::Boolean { .. } => ftd::p2::Kind::boolean(),
+            Value::Record { name: id, .. } => ftd::p2::Kind::Record {
                 name: id.to_string(),
             },
-            Value::OrType { name: id, .. } => crate::p2::Kind::OrType {
+            Value::OrType { name: id, .. } => ftd::p2::Kind::OrType {
                 name: id.to_string(),
             },
-            Value::List { kind, .. } => crate::p2::Kind::List {
+            Value::List { kind, .. } => ftd::p2::Kind::List {
                 kind: Box::new(kind.to_owned()),
             },
-            Value::Map { kind, .. } => crate::p2::Kind::Map {
+            Value::Map { kind, .. } => ftd::p2::Kind::Map {
                 kind: Box::new(kind.to_owned()),
             },
+            Value::UI { kind, .. } => kind.to_owned(),
         }
     }
 
@@ -415,11 +440,15 @@ impl Value {
 }
 
 impl Variable {
-    pub fn list_from_p1(p1: &crate::p1::Section, doc: &crate::p2::TDoc) -> crate::p1::Result<Self> {
-        let var_data =
-            ftd::variable::VariableData::get_name_kind(&p1.name, doc.name, p1.line_number, true)?;
+    pub fn list_from_p1(p1: &ftd::p1::Section, doc: &ftd::p2::TDoc) -> ftd::p1::Result<Self> {
+        let var_data = ftd::variable::VariableData::get_name_kind(
+            &p1.name,
+            doc,
+            p1.line_number,
+            vec![].as_slice(),
+        )?;
         let name = doc.resolve_name(p1.line_number, &var_data.name)?;
-        let kind = crate::p2::Kind::for_variable(p1.line_number, &p1.name, None, doc, None, true)?;
+        let kind = ftd::p2::Kind::for_variable(p1.line_number, &p1.name, None, doc, None)?;
         if !kind.is_list() {
             return ftd::e2(
                 format!("Expected list found: {:?}", p1),
@@ -437,7 +466,7 @@ impl Variable {
         })
     }
 
-    pub fn map_from_p1(p1: &crate::p1::Section, doc: &crate::p2::TDoc) -> crate::p1::Result<Self> {
+    pub fn map_from_p1(p1: &ftd::p1::Section, doc: &ftd::p2::TDoc) -> ftd::p1::Result<Self> {
         let name = doc.resolve_name(
             p1.line_number,
             ftd::get_name("map", p1.name.as_str(), doc.name)?,
@@ -446,7 +475,7 @@ impl Variable {
             name,
             value: Value::Map {
                 data: Default::default(),
-                kind: crate::p2::Kind::from(
+                kind: ftd::p2::Kind::from(
                     p1.line_number,
                     p1.header.str(doc.name, p1.line_number, "type")?,
                     doc,
@@ -459,21 +488,21 @@ impl Variable {
 
     pub fn update_from_p1(
         &mut self,
-        p1: &crate::p1::Section,
-        doc: &crate::p2::TDoc,
-    ) -> crate::p1::Result<()> {
+        p1: &ftd::p1::Section,
+        doc: &ftd::p2::TDoc,
+    ) -> ftd::p1::Result<()> {
         fn read_value(
             line_number: usize,
-            kind: &crate::p2::Kind,
-            p1: &crate::p1::Section,
-            doc: &crate::p2::TDoc,
-        ) -> crate::p1::Result<crate::Value> {
+            kind: &ftd::p2::Kind,
+            p1: &ftd::p1::Section,
+            doc: &ftd::p2::TDoc,
+        ) -> ftd::p1::Result<ftd::Value> {
             Ok(match kind {
-                crate::p2::Kind::Integer { .. } => read_integer(p1, doc.name)?,
-                crate::p2::Kind::Decimal { .. } => read_decimal(p1, doc.name)?,
-                crate::p2::Kind::Boolean { .. } => read_boolean(p1, doc.name)?,
-                crate::p2::Kind::String { .. } => read_string(p1, doc.name)?,
-                crate::p2::Kind::Record { name } => {
+                ftd::p2::Kind::Integer { .. } => read_integer(p1, doc.name)?,
+                ftd::p2::Kind::Decimal { .. } => read_decimal(p1, doc.name)?,
+                ftd::p2::Kind::Boolean { .. } => read_boolean(p1, doc.name)?,
+                ftd::p2::Kind::String { .. } => read_string(p1, doc.name)?,
+                ftd::p2::Kind::Record { name } => {
                     doc.get_record(line_number, name)?.create(p1, doc)?
                 }
                 _ => unimplemented!("{:?}", kind),
@@ -494,7 +523,7 @@ impl Variable {
             (ftd::p2::Kind::Record { name }, _) => {
                 self.value = doc.get_record(p1.line_number, name)?.create(&p1, doc)?
             }
-            (ftd::p2::Kind::List { kind }, crate::Value::List { data, .. }) => {
+            (ftd::p2::Kind::List { kind }, ftd::Value::List { data, .. }) => {
                 data.push(read_value(p1.line_number, kind, &p1, doc)?);
             }
             (ftd::p2::Kind::Map { .. }, _) => {
@@ -506,18 +535,29 @@ impl Variable {
         Ok(())
     }
 
-    pub fn from_p1(p1: &crate::p1::Section, doc: &crate::p2::TDoc) -> crate::p1::Result<Self> {
-        let var_data =
-            ftd::variable::VariableData::get_name_kind(&p1.name, doc.name, p1.line_number, true)?;
+    pub fn from_p1(p1: &ftd::p1::Section, doc: &ftd::p2::TDoc) -> ftd::p1::Result<Self> {
+        let var_data = ftd::variable::VariableData::get_name_kind(
+            &p1.name,
+            doc,
+            p1.line_number,
+            vec![].as_slice(),
+        )?;
+        if !var_data.is_variable() {
+            return ftd::e2(
+                format!("expected variable, found: {}", p1.name),
+                doc.name,
+                p1.line_number,
+            );
+        }
         let name = var_data.name;
-        let value = match var_data.kind.as_deref() {
-            Some("string") => read_string(p1, doc.name)?,
-            Some("integer") => read_integer(p1, doc.name)?,
-            Some("decimal") => read_decimal(p1, doc.name)?,
-            Some("boolean") => read_boolean(p1, doc.name)?,
-            Some(t) => match doc.get_thing(p1.line_number, t)? {
-                crate::p2::Thing::Record(r) => r.create(p1, doc)?,
-                crate::p2::Thing::OrTypeWithVariant { e, variant } => e.create(p1, variant, doc)?,
+        let value = match var_data.kind.as_str() {
+            "string" => read_string(p1, doc.name)?,
+            "integer" => read_integer(p1, doc.name)?,
+            "decimal" => read_decimal(p1, doc.name)?,
+            "boolean" => read_boolean(p1, doc.name)?,
+            t => match doc.get_thing(p1.line_number, t)? {
+                ftd::p2::Thing::Record(r) => r.create(p1, doc)?,
+                ftd::p2::Thing::OrTypeWithVariant { e, variant } => e.create(p1, variant, doc)?,
                 t => {
                     return ftd::e2(
                         format!("unexpected thing found: {:?}", t),
@@ -526,13 +566,6 @@ impl Variable {
                     )
                 }
             },
-            None => {
-                let (caption, is_body) = match p1.caption.as_deref() {
-                    Some(v) => (v.to_string(), false),
-                    None => (p1.body(p1.line_number, doc.name)?, true),
-                };
-                guess_type(&caption, is_body)?
-            }
         };
 
         Ok(Variable {
@@ -544,8 +577,8 @@ impl Variable {
 
     pub fn get_value(
         &self,
-        p1: &crate::p1::Section,
-        doc: &crate::p2::TDoc,
+        p1: &ftd::p1::Section,
+        doc: &ftd::p2::TDoc,
     ) -> ftd::p1::Result<ftd::Value> {
         match self.value.kind() {
             ftd::p2::Kind::String { .. } => read_string(p1, doc.name),
@@ -553,7 +586,7 @@ impl Variable {
             ftd::p2::Kind::Decimal { .. } => read_decimal(p1, doc.name),
             ftd::p2::Kind::Boolean { .. } => read_boolean(p1, doc.name),
             ftd::p2::Kind::Record { name } => match doc.get_thing(p1.line_number, &name)? {
-                crate::p2::Thing::Record(r) => r.create(p1, doc),
+                ftd::p2::Thing::Record(r) => r.create(p1, doc),
                 t => ftd::e2(
                     format!("expected record type, found: {:?}", t),
                     doc.name,
@@ -561,7 +594,7 @@ impl Variable {
                 ),
             },
             ftd::p2::Kind::OrType { name } => match doc.get_thing(p1.line_number, &name)? {
-                crate::p2::Thing::OrTypeWithVariant { e, variant } => e.create(p1, variant, doc),
+                ftd::p2::Thing::OrTypeWithVariant { e, variant } => e.create(p1, variant, doc),
                 t => ftd::e2(
                     format!("expected or-type type, found: {:?}", t),
                     doc.name,
@@ -577,7 +610,7 @@ impl Variable {
     }
 }
 
-pub fn guess_type(s: &str, is_body: bool) -> crate::p1::Result<Value> {
+pub fn guess_type(s: &str, is_body: bool) -> ftd::p1::Result<Value> {
     if is_body {
         return Ok(Value::String {
             text: s.to_string(),
@@ -604,7 +637,7 @@ pub fn guess_type(s: &str, is_body: bool) -> crate::p1::Result<Value> {
     })
 }
 
-fn read_string(p1: &crate::p1::Section, doc_id: &str) -> crate::p1::Result<Value> {
+fn read_string(p1: &ftd::p1::Section, doc_id: &str) -> ftd::p1::Result<Value> {
     match (&p1.caption, &p1.body_without_comment()) {
         (Some(c), Some(b)) => ftd::e2(
             format!("both caption: `{}` and body: `{}` present", c, b.1),
@@ -627,7 +660,7 @@ fn read_string(p1: &crate::p1::Section, doc_id: &str) -> crate::p1::Result<Value
     }
 }
 
-fn read_integer(p1: &crate::p1::Section, doc_id: &str) -> crate::p1::Result<Value> {
+fn read_integer(p1: &ftd::p1::Section, doc_id: &str) -> ftd::p1::Result<Value> {
     let caption = p1.caption(p1.line_number, doc_id)?;
     if let Ok(v) = caption.parse::<i64>() {
         return Ok(Value::Integer { value: v });
@@ -636,7 +669,7 @@ fn read_integer(p1: &crate::p1::Section, doc_id: &str) -> crate::p1::Result<Valu
     ftd::e2("not a valid integer", doc_id, p1.line_number)
 }
 
-fn read_decimal(p1: &crate::p1::Section, doc_id: &str) -> crate::p1::Result<Value> {
+fn read_decimal(p1: &ftd::p1::Section, doc_id: &str) -> ftd::p1::Result<Value> {
     let caption = p1.caption(p1.line_number, doc_id)?;
     if let Ok(v) = caption.parse::<f64>() {
         return Ok(Value::Decimal { value: v });
@@ -645,7 +678,7 @@ fn read_decimal(p1: &crate::p1::Section, doc_id: &str) -> crate::p1::Result<Valu
     ftd::e2("not a valid float", doc_id, p1.line_number)
 }
 
-fn read_boolean(p1: &crate::p1::Section, doc_id: &str) -> crate::p1::Result<Value> {
+fn read_boolean(p1: &ftd::p1::Section, doc_id: &str) -> ftd::p1::Result<Value> {
     let caption = p1.caption(p1.line_number, doc_id)?;
     if let Ok(v) = caption.parse::<bool>() {
         return Ok(Value::Boolean { value: v });
@@ -657,8 +690,9 @@ fn read_boolean(p1: &crate::p1::Section, doc_id: &str) -> crate::p1::Result<Valu
 #[derive(Debug, Clone)]
 pub struct VariableData {
     pub name: String,
-    pub kind: Option<String>,
+    pub kind: String,
     pub modifier: VariableModifier,
+    pub type_: Type,
 }
 
 #[derive(Debug, Clone)]
@@ -668,23 +702,40 @@ pub enum VariableModifier {
     Optional,
 }
 
+#[derive(Debug, Clone)]
+pub enum Type {
+    Variable,
+    Component,
+}
+
 impl VariableData {
     pub fn get_name_kind(
         s: &str,
-        doc_id: &str,
+        doc: &ftd::p2::TDoc,
         line_number: usize,
-        is_variable: bool,
+        var_types: &[String],
     ) -> ftd::p1::Result<VariableData> {
-        let expr = s.split_whitespace().collect::<Vec<&str>>();
-        if expr.len() > 4 {
+        if s.starts_with("record ")
+            || s.starts_with("or-type ")
+            || s.starts_with("map ")
+            || s == "container"
+        {
             return ftd::e2(
-                format!("invalid variable or list declaration, found: `{}`", s),
-                doc_id,
+                format!("invalid declaration, found: `{}`", s),
+                doc.name,
                 line_number,
             );
         }
-        let mut name = expr.get(0);
-        let mut kind = None;
+        let expr = s.split_whitespace().collect::<Vec<&str>>();
+        if expr.len() > 4 || expr.len() <= 1 {
+            return ftd::e2(
+                format!("invalid declaration, found: `{}`", s),
+                doc.name,
+                line_number,
+            );
+        }
+        let mut name = expr.get(1);
+        let mut kind = expr.get(0).map(|k| k.to_string());
         let mut modifier = VariableModifier::None;
         if expr.len() == 4 {
             if expr.get(1).unwrap().eq(&"or") {
@@ -693,7 +744,7 @@ impl VariableData {
             } else {
                 return ftd::e2(
                     format!("invalid variable or list declaration, found: `{}`", s),
-                    doc_id,
+                    doc.name,
                     line_number,
                 );
             }
@@ -709,52 +760,49 @@ impl VariableData {
             } else {
                 return ftd::e2(
                     format!("invalid variable or list declaration, found: `{}`", s),
-                    doc_id,
+                    doc.name,
                     line_number,
                 );
             }
-        } else if expr.len() == 2 {
-            name = expr.get(1);
-            kind = expr.get(0).map(|k| k.to_string());
         }
 
-        let var_name = {
-            let error = ftd::e2(
-                if is_variable {
-                    format!(
-                        "list name should be present and should start with $, found: `{}`",
-                        s
-                    )
-                } else {
-                    format!("name should not start with $, found: `{}`", s)
-                },
-                doc_id,
-                line_number,
-            );
-            if name.is_none() {
-                return error;
-            }
+        let var_kind = kind.ok_or(ftd::p1::Error::ParseError {
+            message: format!("kind not found `{}`", s),
+            doc_id: doc.name.to_string(),
+            line_number,
+        })?;
 
-            let name = name.unwrap();
-            let mut var_name = name.to_string();
-            if let Some(n) = name.strip_prefix('$') {
-                if !is_variable {
-                    return error;
-                }
-                var_name = n.to_string();
-            } else if is_variable {
-                return error;
+        let type_ = match var_kind.as_str() {
+            "string" | "caption" | "body" | "body or caption" | "caption or body" | "integer"
+            | "decimal" | "boolean" => Type::Variable,
+            a if doc.get_record(line_number, a).is_ok()
+                || doc.get_or_type(line_number, a).is_ok()
+                || doc.get_or_type_with_variant(line_number, a).is_ok()
+                || var_types.contains(&a.to_string()) =>
+            {
+                Type::Variable
             }
-            var_name
+            _ => Type::Component,
         };
 
         Ok(VariableData {
-            name: var_name,
-            kind,
+            name: name
+                .ok_or(ftd::p1::Error::ParseError {
+                    message: format!("name not found `{}`", s),
+                    doc_id: doc.name.to_string(),
+                    line_number,
+                })?
+                .to_string(),
+            kind: var_kind,
             modifier,
+            type_,
         })
     }
 
+    pub fn is_variable(&self) -> bool {
+        matches!(self.type_, Type::Variable)
+    }
+
     pub fn is_none(&self) -> bool {
         matches!(self.modifier, VariableModifier::None)
     }
@@ -769,17 +817,17 @@ impl VariableData {
 }
 #[cfg(test)]
 mod test {
-    use crate::test::*;
+    use ftd::test::*;
 
     macro_rules! p2 {
         ($s:expr, $n: expr, $v: expr, $c: expr,) => {
             p2!($s, $n, $v, $c)
         };
         ($s:expr, $n: expr, $v: expr, $c: expr) => {
-            let p1 = crate::p1::parse(indoc::indoc!($s), "foo").unwrap();
+            let p1 = ftd::p1::parse(indoc::indoc!($s), "foo").unwrap();
             let mut bag = std::collections::BTreeMap::new();
             let aliases = std::collections::BTreeMap::new();
-            let mut d = crate::p2::TDoc {
+            let mut d = ftd::p2::TDoc {
                 name: "foo",
                 bag: &mut bag,
                 aliases: &aliases,
@@ -798,65 +846,58 @@ mod test {
     #[test]
     fn int() {
         use super::Value::Integer;
-        p2!("-- $x: 10", "x", Integer { value: 10 }, vec![],);
-        p2!("-- integer $x: 10", "x", Integer { value: 10 }, vec![],);
+        p2!("-- integer x: 10", "x", Integer { value: 10 }, vec![],);
     }
 
     #[test]
     fn float() {
         use super::Value::Decimal;
-        p2!("-- $x: 10.0", "x", Decimal { value: 10.0 }, vec![],);
-        p2!("-- decimal $x: 10", "x", Decimal { value: 10.0 }, vec![],);
+        p2!("-- decimal x: 10", "x", Decimal { value: 10.0 }, vec![],);
     }
 
     #[test]
     fn bool() {
         use super::Value::Boolean;
-        p2!("-- $x: true", "x", Boolean { value: true }, vec![],);
-        p2!(
-            "-- boolean $x: false",
-            "x",
-            Boolean { value: false },
-            vec![],
-        );
+        p2!("-- boolean x: true", "x", Boolean { value: true }, vec![],);
+        p2!("-- boolean x: false", "x", Boolean { value: false }, vec![],);
     }
 
     #[test]
     fn str() {
         use super::Value::String;
         p2!(
-            "-- $x: hello",
+            "-- string x: hello",
             "x",
             String {
                 text: "hello".to_string(),
-                source: crate::TextSource::Caption
+                source: ftd::TextSource::Caption
             },
             vec![],
         );
         p2!(
-            "-- $x:\n\nhello world\nyo!",
+            "-- string x:\n\nhello world\nyo!",
             "x",
             String {
                 text: "hello world\nyo!".to_string(),
-                source: crate::TextSource::Body
+                source: ftd::TextSource::Body
             },
             vec![],
         );
         p2!(
-            "-- string $x: 10",
+            "-- string x: 10",
             "x",
             String {
                 text: "10".to_string(),
-                source: crate::TextSource::Caption
+                source: ftd::TextSource::Caption
             },
             vec![],
         );
         p2!(
-            "-- string $x: true",
+            "-- string x: true",
             "x",
             String {
                 text: "true".to_string(),
-                source: crate::TextSource::Caption
+                source: ftd::TextSource::Caption
             },
             vec![],
         );
@@ -868,11 +909,11 @@ mod test {
         let mut bag = default_bag();
         bag.insert(
             s("foo/bar#pull-request"),
-            crate::p2::Thing::Record(crate::p2::Record {
+            ftd::p2::Thing::Record(ftd::p2::Record {
                 name: s("foo/bar#pull-request"),
                 fields: std::array::IntoIter::new([
-                    (s("title"), crate::p2::Kind::caption()),
-                    (s("about"), crate::p2::Kind::body()),
+                    (s("title"), ftd::p2::Kind::caption()),
+                    (s("about"), ftd::p2::Kind::body()),
                 ])
                 .collect(),
                 instances: Default::default(),
@@ -881,34 +922,34 @@ mod test {
 
         bag.insert(
             "foo/bar#pr".to_string(),
-            crate::p2::Thing::Variable(crate::Variable {
+            ftd::p2::Thing::Variable(ftd::Variable {
                 name: "foo/bar#pr".to_string(),
-                value: crate::Value::List {
-                    data: vec![crate::Value::Record {
+                value: ftd::Value::List {
+                    data: vec![ftd::Value::Record {
                         name: s("foo/bar#pull-request"),
                         fields: std::array::IntoIter::new([
                             (
                                 s("title"),
-                                crate::PropertyValue::Value {
-                                    value: crate::Value::String {
+                                ftd::PropertyValue::Value {
+                                    value: ftd::Value::String {
                                         text: "some pr".to_string(),
-                                        source: crate::TextSource::Caption,
+                                        source: ftd::TextSource::Caption,
                                     },
                                 },
                             ),
                             (
                                 s("about"),
-                                crate::PropertyValue::Value {
-                                    value: crate::Value::String {
+                                ftd::PropertyValue::Value {
+                                    value: ftd::Value::String {
                                         text: "yo yo".to_string(),
-                                        source: crate::TextSource::Body,
+                                        source: ftd::TextSource::Body,
                                     },
                                 },
                             ),
                         ])
                         .collect(),
                     }],
-                    kind: crate::p2::Kind::Record {
+                    kind: ftd::p2::Kind::Record {
                         name: s("foo/bar#pull-request"),
                     },
                 },
@@ -922,9 +963,8 @@ mod test {
             caption title:
             body about:
 
-            -- component pr-view:
-            pull-request $pr:
-            component: ftd.column
+            -- ftd.column pr-view:
+            pull-request pr:
 
             --- ftd.text:
             text: $pr.title
